/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the UserStyles extension for Firebird
 *
 * The Initial Developer of the Original Code is Stephen Clavering <mozilla@clav.co.uk>
 *
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/* User Styles Extension
 *
 * This extension is effectively an extension to the StyleSelector extension.  It adds
 * a context menu to StyleSelector's Style menu in the status bar, on which is listed
 * a set of style sheets which can be specified in list.js in a fairly obvious form.
 *
 * Both style sheets put in the /chrome/userstyles/ folder, and ones with an http uri
 * should work fine.
 *
 * Currently the form of list.js only supports flat menus, though seperators can be
 * inserted by adding a "null" entry at the desired point.  Future versions might allow
 * for hierarchial menus.
 *
 * User style sheets are added to a page by creating a new <link/> element. The title is left
 * blank, and the <link> element has isUserStyle = true, and userStyleName = {what the title
 * would be}.  StyleSelector will ignore sheets where the ownerNode has the first property set.
 * It would be nice to store these properties on the DOMStyleSheet itself, but that doesn't
 * seem to work, reasons unknown.
 *
 * At the moment this probably only works with HTML documents - I'm not even sure about XHTML
 * though I expect that's OK.  Could extend it to work on generic XML, either by adding
 * xml-stylesheet P.I.s, or byt using createElementNS to add a <html:link> element.
 */


const UserStyles = {
  // initUserStylesMenu is called in onpopupshowing, and calls createUserStylesMenu the first time it runs
  menuCreated: false,

  createMenu: function(menu) {
    for(var i = 0; i < UserStylesList.length; i++) {
      // use null in list to add separator :)
      if(!UserStylesList[i]) {
        menu.appendChild(document.createElement("menuseparator"));
      } else {
        var mi = document.createElement("menuitem");
        mi.setAttribute("type","checkbox");
        mi.setAttribute("label",UserStylesList[i][0]);
        mi.setAttribute("value",UserStylesList[i][1]);
        menu.appendChild(mi);
      }
    }
    this.menuCreated = true;
  },

  initMenu: function(event, menu, frame) {
    if(!this.menuCreated) this.createMenu(menu);
    var doc = frame.document;
    //XXX this should be changed to allow xml docs?
    if(!doc || !doc instanceof Components.interfaces.nsIDOMHTMLDocument) return;
    var sheets = doc.styleSheets;
    // toggle the checkboxes on/off appropriately
    for(var mi = menu.firstChild; mi; mi = mi.nextSibling) {
      if(mi.localName!="menuitem") continue;
      if(this.styleInUse(sheets,mi.label)) mi.setAttribute("checked","true");
      else mi.removeAttribute("checked");
    }
  },
  styleInUse: function(sheets, label) {
    for(var i = 0; i < sheets.length; i++) {
      var node = sheets[i].ownerNode;
      if(node && node.isUserStyle && node.userStyleName==label)
        return true;
    }
    return false;
  },

  toggleUserStyle: function(event, menu, frame) {
    var t = event.target;
    // checked attr reflects state *after* click.
    // hmm. why isn't ".checked" in the toolkit?!?
    if(t.getAttribute("checked")=="true")
      this.addUserStyle(t.label,t.value,frame);
    else
      this.removeUserStyle(t.label,t.value,frame);
  },

  // user styles are toggled on/off, so we don't need to check if it's there already
  addUserStyle: function(label, href, frame) {
    var link = frame.document.createElement("link");
    link.href = href;
    link.rel = "stylesheet";
    link.type = "text/css";
    // tried adding these to link.sheet but it doesn't work
    link.isUserStyle = true;
    link.userStyleName = label;
    // first node could be a comment or something like that
    // what will this do on tag soup with no <head/> ?
    frame.document.getElementsByTagName("head")[0].appendChild(link);
    // add to subframes
    for(var i = 0; i < frame.frames.length; i++)
      this.addUserStyle(label, href, frame.frames[i]);
  },
  removeUserStyle: function(label, href, frame) {
    var sheets = frame.document.styleSheets;
    for(var i = 0; i < sheets.length; i++) {
      var sheetNode = sheets[i].ownerNode;
      if(sheetNode && sheetNode.isUserStyle && sheetNode.userStyleName==label)
        sheetNode.parentNode.removeChild(sheetNode);
    }
    // deal with subframes
    for(var i = 0; i < frame.frames.length; i++)
      this.removeUserStyle(label, href, frame.frames[i]);
  }
}

